from openUtils import OpenUtils


def tif2tif():
    myOpenUtils = OpenUtils()
    myOpenUtils.initParams(
        "/Volumes/pioneer/gdal_Demo/translateResult/Z_NAFP_C_BABJ_20180701091809_P_CLDAS_RT_ASI_0P0625_HOR-PRS-2018070109.tif",
        file_type="GeoTiff",
        out_file="./tif2tif.tif",
        export_type="GeoTiff",
        data_type='float32',
        lat_order="asc",
        values_strs=["lats", "lons", "abc"],
        nc_values=["LAT", "LON", "PAIR"],
        is_rewirte_data=False,
        proj="mercator")

    print myOpenUtils.lats
def nc2nc():
    myOpenUtils = OpenUtils()
    myOpenUtils.initParams(
        "/Volumes/pioneer/gdal_Demo/cldas_/Z_NAFP_C_BABJ_20180701091809_P_CLDAS_RT_ASI_0P0625_HOR-PRS-2018070109.nc",
        file_type="nc",
        out_file="./nc2nc.nc",
        export_type="nc",
        data_type='float32',
        values_strs=["lats", "lons", "nc2nc"],
        nc_values=["LAT", "LON", "PAIR"],
        proj="mercator")


def img2img():
    myOpenUtils = OpenUtils()
    myOpenUtils.initParams(
        "/Volumes/pioneer/img_hdr/FY3C_L_2016_08_29_11_28_A_G_VIRRX_L1B.img",
        file_type="img",
        out_file="./img2img.img",
        export_type="img",
        data_type='float32',
        # lat_order="asc",
        # data_order="desc",
        values_strs=["lats", "lons", "img2img"],
        nc_values=["LAT", "LON", "PAIR"],
        proj="mercator")


def girb2_girb2():
    myOpenUtils = OpenUtils()
    myOpenUtils.initParams(
        "/Volumes/pioneer/liaoning/grb2/20180806/Z_SURF_C_BABJ_20180806001030_P_CMPA_FAST_CHN_0P05_HOR-PRE-2018080600.GRB2",
        # "./utm.grb2",
        file_type="grib2",
        out_file="./grib2_2_grib2.grb2",
        export_type="grib2",
        data_type='float32',
        proj="mercator")


def nc2tif():
    myOpenUtils = OpenUtils()
    myOpenUtils.initParams(
        "/Volumes/pioneer/gdal_Demo/cldas_/Z_NAFP_C_BABJ_20180701091809_P_CLDAS_RT_ASI_0P0625_HOR-PRS-2018070109.nc",
        file_type="nc",
        out_file="./nc2tif.tif",
        export_type="GeoTiff",
        data_type='float32',
        values_strs=["lats", "lons", "abc"],
        nc_values=["LAT", "LON", "PAIR"],
        proj="mercator")


def tif2nc():
    myOpenUtils = OpenUtils()
    myOpenUtils.initParams(
        "/Volumes/pioneer/gdal_Demo/translateResult/Z_NAFP_C_BABJ_20180701091809_P_CLDAS_RT_ASI_0P0625_HOR-PRS-2018070109.tif",
        file_type="GeoTiff",
        out_file="./tif2nc.nc",
        export_type="nc",
        data_type='float32',
        lat_order="asc",
        values_strs=["lats", "lons", "tif2nc"],
        nc_values=["LAT", "LON", "PAIR"],
        proj="mercator")


def nc2img():
    myOpenUtils = OpenUtils()
    myOpenUtils.initParams(
        "/Volumes/pioneer/gdal_Demo/cldas_/Z_NAFP_C_BABJ_20180701091809_P_CLDAS_RT_ASI_0P0625_HOR-PRS-2018070109.nc",
        file_type="nc",
        out_file="./nc2img.img",
        export_type="img",
        data_type='float32',
        lat_order="asc",
        data_order="desc",
        values_strs=["lats", "lons", "abc"],
        nc_values=["LAT", "LON", "PAIR"],
        proj="mercator")


def img2nc():
    myOpenUtils = OpenUtils()
    myOpenUtils.initParams(
        "/Volumes/pioneer/img_hdr/FY3C_L_2016_08_29_11_28_A_G_VIRRX_L1B.img",
        file_type="img",
        out_file="./img2nc.nc",
        export_type="nc",
        data_type='float32',
        # lat_order="asc",
        # data_order="desc",
        values_strs=["lats", "lons", "img2nc_0", "img2nc_1", "img2nc_2", "img2nc_3", "img2nc_4", "img2nc_5", "img2nc_6",
                     "img2nc_7", "img2nc_8", "img2nc_9"],
        nc_values=["LAT", "LON", "PAIR"],
        proj="mercator")


def tif2img():
    myOpenUtils = OpenUtils()
    myOpenUtils.initParams(
        "/Volumes/pioneer/gdal_Demo/translateResult/Z_NAFP_C_BABJ_20180701091809_P_CLDAS_RT_ASI_0P0625_HOR-PRS-2018070109.tif",
        file_type="GeoTiff",
        out_file="./tif2img.img",
        export_type="img",
        data_type='float32',
        lat_order="asc",
        values_strs=["lats", "lons", "tif2img"],
        nc_values=["LAT", "LON", "PAIR"],
        proj="mercator")


def img2tif():
    myOpenUtils = OpenUtils()
    myOpenUtils.initParams(
        "/Volumes/pioneer/img_hdr/FY3C_L_2016_08_29_11_28_A_G_VIRRX_L1B.img",
        file_type="img",
        out_file="./img2tif_asc.tif",
        export_type="GeoTiff",
        data_type='float32',
        # lat_order="asc",
        # data_order="desc",
        values_strs=["lats", "lons", "img2tif"],
        nc_values=["LAT", "LON", "PAIR"],
        proj="mercator")


if __name__ == '__main__':
    tif2tif()
    print "0-----"
    # nc2nc()
    # print "1-----"
    # img2img()
    # print "8-----"
    # nc2tif()
    # print "2-----"
    # tif2nc()
    # print "3-----"
    # nc2img()
    # print "4-----"
    # img2nc()
    # print "5-----"
    # tif2img()
    # print "6-----"
    # img2tif()
    # print "7-----"
    # girb2_girb2()
    # print "8-----"



# -*- coding:utf-8 -*-
from osgeo import gdal
import geotiffreader
import numpy as np
import os


def read(currentGrb2File):
    driver = gdal.GetDriverByName('GRIB')
    driver.Register()
    dataset = gdal.Open(currentGrb2File, gdal.GA_ReadOnly)
    x_size = dataset.RasterXSize
    y_size = dataset.RasterYSize
    geotrans = dataset.GetGeoTransform()  # 仿射矩阵
    proj = dataset.GetProjection()  # 地图投影信息
    bands = dataset.RasterCount  # 获取文件纵向深度（几个通道）
    in_data = []
    no_data = []
    (in_lats, in_lons) = geotiffreader.createXY(geotrans, x_size, y_size)
    for band in range(bands):  # 以下是循环遍历读取每一层数据
        currentBand = dataset.GetRasterBand(band + 1)
        current_data = currentBand.ReadAsArray(0, 0, x_size, y_size)
        current_nodata = currentBand.GetNoDataValue()
        in_data.append(current_data)
        no_data.append(current_nodata)
    in_data = np.array(in_data)
    del dataset
    return (geotrans, proj, in_lats, in_lons, in_data, no_data)


def wirte(lat, lon, data, nodata, export_file, order, proj, exportType):
    print data.shape
    print nodata
    if 'int8' in data.dtype.name:  # 注意！！！此处的数据类型一定要注意，如果源数据数据类型和写入法人设置不一样，致命的疏忽
        datatype = gdal.GDT_Byte
    elif 'int16' in data.dtype.name:
        datatype = gdal.GDT_Int16
    elif 'float32' in data.dtype.name:
        datatype = gdal.GDT_Float32
    else:
        datatype = gdal.GDT_Float64
    # 判读数组维数
    if len(data.shape) == 3:
        im_bands, im_height, im_width = data.shape
    else:
        im_bands, (im_height, im_width) = 1, data.shape
    gdal.AllRegister()
    driver = gdal.GetDriverByName('GRIB')
    drimen = gdal.GetDriverByName("MEM")

    dout = drimen.Create(export_file, im_width, im_height, im_bands, datatype)
    if (os.path.splitext(export_file)[-1] == ".grb2"):
        pass
    else:
        print "export_file name error"
        return
    nodata = np.asarray(nodata, dtype="double")
    local_transform = geotiffreader.createGeotransform(lat, lon, order)
    dout.SetGeoTransform(local_transform)  # 写入仿射变换参数
    srs = geotiffreader.createSrs(proj)
    if (srs != None):
        dout.SetProjection(srs)  # 写入投影
    else:
        print "input srs/proj error"
    print "nodata", nodata

    if im_bands == 1:
        # dout.WriteRaster(0, 0, im_width, im_height, data[0], buf_type=gdal.GDT_Float32)
        dout.GetRasterBand(1).WriteArray(data[0])
        if (nodata == None or nodata.__len__() == 0):
            pass
        else:
            if (nodata[0] != None):
                pass
                # dout.GetRasterBand(1).SetNoDataValue(nodata[0])  # 设置无效值
    else:
        for i in range(im_bands):
            dout.GetRasterBand(i + 1).WriteArray(data[i])
            if (nodata.__len__() == 0):
                pass
            else:
                if (nodata[i] != None):
                    pass
                    # dout.GetRasterBand(1).SetNoDataValue(nodata[i])  # 设置无效值
    options = [
        # 'DATA_ENCODING=' + data_encoding,
        # 'PDS_PDTN=8',
        # 'PDS_TEMPLATE_NUMBERS=0 5 2 0 0 0 255 255 1 0 0 0 43 1 0 0 0 0 0 255 129 255 255 255 255 7 216 2 23 12 0 0 1 0 0 0 0 3 255 1 0 0 0 12 1 0 0 0 0'
        # 'GRIB_PDS_TEMPLATE_ASSEMBLED_VALUES=1 8 0 0 0 0 0 1 0 1 0 0 255 0 0 2018 8 6 0 0 0 1 0 1 2 1 1 1 0'
        # 'GRIB_PDS_TEMPLATE_NUMBERS = 1 8 0 0 0 0 0 0 1 0 0 0 0 1 0 0 0 0 0 255 0 0 0 0 0 7 226 8 6 0 0 0 1 0 0 0 0 1 2 1 0 0 0 1 1 0 0 0 0'
    ]
    print "====="
    driver.CreateCopy(export_file, dout)
    del drimen



# -*- coding: utf-8 -*-
import matplotlib.pyplot as plt


def line(dates, LSTPoints, obsPoints, siteCode, name):
    newLSTPoints = []
    newLSTx = []
    for index, LSTPoint in enumerate(LSTPoints):
        if (LSTPoint != 0):
            newLSTPoints.append(LSTPoint)
            newLSTx.append(index)
    print newLSTPoints
    print newLSTx
    x = range(0, len(dates))
    # 设置图片的大小
    fig = plt.figure(figsize=(50, 20), dpi=80)

    ax = fig.add_subplot(1, 1, 1)

    # 绘制图片
    ax.plot(x, obsPoints)
    ax.plot(newLSTx, newLSTPoints)
    #
    ax.plot(dates, obsPoints, 'go-', label=u'obs', color="b")
    ax.plot(newLSTx, newLSTPoints, 'go-', label=u'hdf', color="r")
    legend = ax.legend(loc='upper right', shadow=False)

    # 设置x州的刻度
    # 调整x轴的刻度
    # 取步长，数字和字符一一对应，数据的长度一样
    plt.xticks(list(x)[::1], dates[::1], rotation=45)

    # 设置添加描述信息
    plt.xlabel('TIME')
    plt.ylabel('TMP')
    plt.title("%s" % siteCode)

    # 展示图片
    plt.savefig("./result/%s_%s.png" % (name, siteCode))


def testDraw(data):
    fig = plt.figure(figsize=(50, 20), dpi=80)
    ax = fig.add_subplot(1, 1, 1)
    cs = ax.imshow(data, cmap="jet")
    plt.colorbar(cs)
    plt.show()



# -*- coding:utf-8 -*-
from osgeo import gdal
from osgeo import osr
from osgeo.gdalconst import *
import os
import numpy as np


# read GeoTiff and  ENVI/img

def read(gtif_file, dataType):
    if (not os.path.exists(gtif_file)):
        print "%s is not exist" % gtif_file
        return;
    if (dataType == "GeoTiff"):
        driver = gdal.GetDriverByName('GTiff')
    elif (dataType == "img"):
        driver = gdal.GetDriverByName('HFA')
    elif (dataType == "grib2"):
        driver = gdal.GetDriverByName('GRIB')
    driver.Register()
    inDs = gdal.Open(gtif_file, GA_ReadOnly)  # 打开文件
    if inDs is None:
        print "file error"
        return;
    else:
        cols = inDs.RasterXSize  # 获取文件的列数
        rows = inDs.RasterYSize  # 获取文件的行数
    bands = inDs.RasterCount  # 获取文件纵向深度（几个通道）
    in_geotransf = inDs.GetGeoTransform()  # 获取放射矩阵
    in_proj = inDs.GetProjection()  # 地图投影信息
    in_data = []
    no_data = []
    (in_lats, in_lons) = createXY(in_geotransf, cols, rows)
    for band in range(bands):  # 以下是循环遍历读取每一层数据
        currentBand = inDs.GetRasterBand(band + 1)
        current_data = currentBand.ReadAsArray(0, 0, cols, rows)
        current_nodata = currentBand.GetNoDataValue()
        in_data.append(current_data)
        no_data.append(current_nodata)
    in_data = np.array(in_data)
    del inDs
    return (in_geotransf, in_proj, in_lats, in_lons, in_data, no_data)


def wirte(lat, lon, data, nodata, export_file, order, proj, exportType):
    print data.shape
    if 'int8' in data.dtype.name:  # 注意！！！此处的数据类型一定要注意，如果源数据数据类型和写入法人设置不一样，致命的疏忽
        datatype = gdal.GDT_Byte
    elif 'int16' in data.dtype.name:
        datatype = gdal.GDT_Int16
    elif 'float32' in data.dtype.name:
        datatype = gdal.GDT_Float32
    else:
        datatype = gdal.GDT_Float64
    # 创建文件
    # 数据类型必须有，因为要计算需要多大内存空间
    if (exportType == "GeoTiff"):
        driver = gdal.GetDriverByName('GTiff')
        if (os.path.splitext(export_file)[-1] == ".tif"):
            pass
        else:
            print "export_file name error"
            return
    elif (exportType == "img"):
        driver = gdal.GetDriverByName('HFA')
        if (os.path.splitext(export_file)[-1] == ".img"):
            pass
        else:
            print "export_file name error"
            return
    elif (exportType == "grib2"):
        driver = gdal.GetDriverByName('GRIB')
        if (os.path.splitext(export_file)[-1] == ".GRB2"):
            pass
        else:
            print "export_file name error"
            return
    driver.Register()
    nodata = np.asarray(nodata, dtype="double")
    # 判读数组维数
    if len(data.shape) == 3:
        im_bands, im_height, im_width = data.shape
    else:
        im_bands, (im_height, im_width) = 1, data.shape
    dataset = driver.Create(export_file, im_width, im_height, im_bands, datatype)
    local_transform = createGeotransform(lat, lon, order)
    print "local_transform", local_transform
    dataset.SetGeoTransform(local_transform)  # 写入仿射变换参数
    srs = createSrs(proj)
    if (srs != None):
        dataset.SetProjection(srs)  # 写入投影
    else:
        print "input srs/proj error"
    print "nodata", nodata
    if im_bands == 1:
        dataset.GetRasterBand(1).WriteArray(data[0])  # 写入数组数据
        if (nodata == None or nodata.__len__() == 0):
            pass
        else:
            dataset.GetRasterBand(1).SetNoDataValue(nodata[0])  # 设置无效值

    else:
        for i in range(im_bands):
            dataset.GetRasterBand(i + 1).WriteArray(data[i])
            if (nodata.__len__() == 0):
                pass
            else:
                if (nodata[i] != None):
                    dataset.GetRasterBand(1).SetNoDataValue(nodata[i])  # 设置无效值
    del dataset


def createGeotransform(lat, lon, order):
    print "order", order
    if (order == "asc"):  # 顺序
        local_transform = (
            min(lon), (max(lon) - min(lon)) / lon.__len__(), 0.0, max(lat), 0.0,
            (min(lat) - max(lat)) / lat.__len__())  # 第二个0.0是读取数据的顺序如果为0.0就按照数据原始行顺序读取，如果为 - 0.0就是按照行逆序读取。
    elif (order == "desc"):  # 倒序
        local_transform = (
            min(lon), (max(lon) - min(lon)) / lon.__len__(), 0.0, min(lat), -0.0,
            abs((min(lat) - max(lat)) / lat.__len__()))  # 第二个0.0是读取数据的顺序如果为0.0就按照数据原始行顺序读取，如果为 - 0.0就是按照行逆序读取。
    return local_transform


def createXY(transform, xSize, ySize):
    lat = np.linspace(transform[5] * ySize + transform[3], transform[3], ySize)
    lon = np.linspace(transform[0], transform[1] * xSize + transform[0], xSize)
    lat = list(lat)
    lat.reverse()
    return (lat, lon)


def createSrs(projstr):
    proj = None
    if (projstr == "mercator"):
        srs4326 = osr.SpatialReference()
        srs4326.ImportFromEPSG(4326)
    proj = str(srs4326)
    return proj



__version__ = '$Id: umOpener.py 27349 2014-05-16 18:58:51Z rouault $'
type_list = ('nc', 'grib2', 'img', 'GeoTiff')
export_list = ('nc', 'grib2', 'img', 'GeoTiff')
datatype_list = ('int8', 'int16', 'float32', 'float64', 'float128')
order = ("asc", "desc")
proj = ("mercator",)
read_file_error = "read file error"
import numpy as np


class OpenUtils():
    def __init__(self):
        pass

    def initParams(self, inputFile, **kwargs):
        arguments = []
        for kwarg_key in kwargs.keys():
            arguments.append("--%s" % kwarg_key)
            arguments.append(kwargs[kwarg_key])
        self.inputFile = inputFile
        self.nc_attrs = None
        self.optparse_init()
        (self.options, self.args) = self.parser.parse_args(args=arguments)
        self.no_data = self.options.nodata
        self.stopped = False
        self.process()

    # -------------------------------------------------------------------------

    def optparse_init(self):
        """Prepare the option parser for input (argv)"""
        from optparse import OptionParser, OptionGroup
        usage = 'Usage: %prog [options] input_file(s) [output]'
        p = OptionParser(usage, version='%prog ' + __version__)
        p.add_option(
            '-t',
            '--file_type',
            dest='intPutType',
            type='choice',
            choices=type_list,
            help="Tile cutting profile (%s) - default 'mercator' (Google Maps compatible)"
                 % ','.join(type_list),
        )
        p.add_option(
            '-o',
            '--out_file',
            dest='outFile',
            help="out_file", )
        p.add_option(
            '-e',
            '--export_type',
            dest='exportType',
            type='choice',
            choices=export_list,
            help='export file type(%s)' % ",".join(export_list)
        )
        p.add_option(
            '-d',
            '--data_type',
            dest='dataType',
            type='choice',
            choices=datatype_list,
            help='output file data type'
        )
        p.add_option(
            '-l',
            '--lat_order',
            dest="latOrder",
            type='choice',
            choices=order,
            help='lat list order'
        )
        p.add_option(
            '--data_order',
            dest="dataOrder",
            type='choice',
            choices=order,
            help='data list order'
        )
        p.add_option(
            '-p',
            '--proj',
            dest="proj",
            type='choice',
            choices=proj,
            help='data set proj'
        )
        p.add_option(
            '-n',
            '--nc_values',
            dest='ncValues',
            help='intput every band name',
        )
        p.add_option(
            '-v',
            '--values_strs',
            dest='valueStrs',
            help='output every band name',
        )
        p.add_option(
            '-m',
            '--miss_value',
            dest='nodata',
            help='export nodata',
        )
        p.add_option(
            '-r',
            '--is_rewirte_data',
            dest="isReWirteData",
            help='return data'
        )
        p.set_defaults(
            exportType='GeoTiff',
            dataType="float32",
            latOrder="asc",
            dataOrder="asc",
            proj="mercator",
            isReWirteData=False,
            nodata=None
        )

        self.parser = p

    # =================================================================================================
    def openFile(self):
        if (self.options.intPutType == "nc"):
            print "read nc "
            import netcdf4reader
            print self.options.ncValues
            print self.options.ncValues[0]
            try:
                lats, lons, nc_data, no_data, lat_attr, lon_attr, nc_attrs = netcdf4reader.read(self.inputFile,
                                                                                                self.options.ncValues[
                                                                                                    0],
                                                                                                self.options.ncValues[
                                                                                                    1],
                                                                                                self.options.ncValues[
                                                                                                2:],
                                                                                                self.options.dataType)
                self.lats = lats
                self.lons = lons
                if (self.options.dataOrder == "asc"):
                    self.data = nc_data
                else:
                    reverse_data = []
                    for index in range(nc_data.shape[0]):
                        reverse_data.append(nc_data[0][::-1])
                    self.data = np.array(reverse_data)
                self.nc_attrs = nc_attrs
                self.lat_attr = lat_attr
                self.lon_attr = lon_attr
                self.no_data = no_data
            except Exception, e:
                self.stop()
                print read_file_error
        elif (self.options.intPutType == "GeoTiff" or self.options.intPutType == "img"):
            print "read gtif/img"
            import geotiffreader
            try:
                (in_geotransf, in_proj, in_lats, in_lons, in_data, no_data) = geotiffreader.read(self.inputFile,
                                                                                                 self.options.intPutType)
                self.lats = in_lats
                self.lons = in_lons
                if (self.options.dataOrder == "asc"):
                    self.data = in_data
                else:
                    reverse_data = []
                    for index in range(in_data.shape[0]):
                        reverse_data.append(in_data[0][::-1])
                    self.data = np.array(reverse_data)
                self.no_data = no_data
                self.nc_attrs = None
                self.lat_attr = None
                self.lon_attr = None
            except Exception, e:
                self.stop()
                print read_file_error
        elif (self.options.intPutType == "grib2"):
            print "read grib/grib2"
            import grib2reader
            try:
                (in_geotransf, in_proj, in_lats, in_lons, in_data, no_data) = grib2reader.read(self.inputFile)
                self.lats = in_lats
                self.lons = in_lons
                if (self.options.dataOrder == "asc"):
                    self.data = in_data
                else:
                    reverse_data = []
                    for index in range(in_data.shape[0]):
                        reverse_data.append(in_data[0][::-1])
                    self.data = np.array(reverse_data)
                self.no_data = no_data
                self.nc_attrs = None
                self.lat_attr = None
                self.lon_attr = None
            except Exception, e:
                self.stop()
                print read_file_error

    # =================================================================================================
    def wirteFile(self):
        if (not self.stopped):
            if (self.options.exportType == "nc"):
                print "create netcdf4"
                import netcdf4reader
                netcdf4reader.wirte(self.options.outFile, self.lats, self.lons, self.data, self.options.valueStrs,
                                    self.lat_attr, self.lon_attr, self.nc_attrs,
                                    self.options.dataType)
            if (self.options.exportType == "GeoTiff" or self.options.exportType == "img"):
                import geotiffreader
                print "self.options.latOrder", self.options.latOrder
                geotiffreader.wirte(self.lats, self.lons, self.data, self.no_data, self.options.outFile,
                                    self.options.latOrder, self.options.proj, self.options.exportType)
            if (self.options.exportType == "grib2"):
                print "create grb2 error"
                # import grib2reader
                # grib2reader.wirte(self.lats, self.lons, self.data, self.no_data, self.options.outFile,
                #                   self.options.latOrder, self.options.proj, self.options.exportType)

        else:
            print "stop wirte file"

    # ==================================================================================================
    def stop(self):
        self.stopped = True

    # ===================================================================================================
    def process(self):
        self.openFile()
        if (self.options.isReWirteData):
            self.wirteFile()



#!/usr/bin/env python
# -*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: neo
# Mail: james_neo@163.com
# Created Time:  2018-11-18
#############################################

from setuptools import setup, find_packages  # 这个包没有的可以pip一下

setup(
    name="umOpener",  # 这里是pip项目发布的名称
    version="0.0.9",  # 版本号，数值大的会优先被pip
    keywords=("pip", "umOpener", "opener"),
    description="netcdf4/GeoTiff/ENVI/grib(only read) read and wirte",
    long_description="""
umOpener
=====

netcdf4/GeoTiff/ENVI/ read and wirte
------------

1. nc<--->nc
2. tif<--->tif
3. img<--->img
4. nc<--->tif
5. nc<--->img
6. tif<--->nc
7. tif<--->img
8. img<--->nc
9. img<--->tif
10.read grib2
""",
    license="MIT Licence",

    url="https://github.com/XiaoXiaowukong/opener",  # 项目相关文件地址，一般是github
    author="neo",
    author_email="james_neo@163.com",

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=['netCDF4', 'numpy']  # 这个项目需要的第三方库
)



# -*- coding:utf-8 -*-
from netCDF4 import Dataset
import os
import sys
import numpy as np


def wirte(output_file, lats, lons, datas, values_strs, lat_attrs, lon_attrs, nc_attrs, data_type):
    nc_dst = Dataset(output_file, 'w', format='NETCDF4')
    lat_nsize = lats.__len__()  # 维度的范围
    lon_nsize = lons.__len__()  # 经度的范围
    x = values_strs[1]
    y = values_strs[0]
    nc_dst.createDimension(y, lat_nsize);
    nc_dst.createDimension(x, lon_nsize);
    nctype = switType(data_type)
    if (datas.__len__() == values_strs[2:].__len__()):
        if nc_attrs != None and "_FillValue" in lat_attrs.keys():
            y_miss_value = nc_attrs[0]["_FillValue"]
            var_value = nc_dst.createVariable(values_strs[0], nctype, (y), fill_value=y_miss_value)
            nc_attrs[0].pop("_FillValue")
        else:
            var_value = nc_dst.createVariable(values_strs[0], nctype, (y))
        try:
            var_value.setncatts(lat_attrs)
        except Exception, e:
            print "lat attr error"
        nc_dst.variables[values_strs[0]][:] = lats
        # ================================================================================================
        if nc_attrs != None and "_FillValue" in lon_attrs.keys():
            x_miss_value = nc_attrs[1]["_FillValue"]
            var_value = nc_dst.createVariable(values_strs[1], nctype, (x), fill_value=x_miss_value)
            nc_attrs[1].pop("_FillValue")
        else:
            var_value = nc_dst.createVariable(values_strs[1], nctype, (x))
        try:
            var_value.setncatts(lon_attrs)
        except Exception, e:
            print "lon attr error"
        nc_dst.variables[values_strs[1]][:] = lons
        # =================================================================================================
        if (nc_attrs != None):
            for values_str, nc_attr, data in zip(values_strs[2:], nc_attrs, datas):
                if nc_attrs != None and "_FillValue" in nc_attr.keys():
                    miss_value = nc_attr['_FillValue']
                    var_value = nc_dst.createVariable(values_str, nctype, (y, x), fill_value=miss_value)
                    nc_attr.pop("_FillValue")
                else:
                    var_value = nc_dst.createVariable(values_str, nctype, (y, x))
                try:
                    var_value.setncatts(nc_attr)
                except Exception, e:
                    print "value attr error"
                nc_dst.variables[values_str][:] = data
        else:
            for values_str, data in zip(values_strs[2:], datas):
                var_value = nc_dst.createVariable(values_str, nctype, (y, x))
                nc_dst.variables[values_str][:] = data
    else:
        print "data is %s" % datas.__len__() + "and values_strs is %s" % values_strs[2:].__len__()
    nc_dst.close()
    del nc_dst


# ===================================================================


def read(input_file, latkey, lonkey, values, dataType):
    if (os.path.exists(input_file)):
        nc_ds = Dataset(input_file)
    else:
        print "%s is not exist" % input_file
        sys.exit()
    nc_data = []
    no_data = []
    nc_attrs = []
    lats = nc_ds.variables[latkey][:]
    lat_attr = {}
    for attr in nc_ds.variables[latkey].ncattrs():
        lat_attr[attr] = getattr(nc_ds.variables[latkey], attr)

    lons = nc_ds.variables[lonkey][:]
    lon_attr = {}
    for attr in nc_ds.variables[lonkey].ncattrs():
        lon_attr[attr] = getattr(nc_ds.variables[lonkey], attr)

    for value in values:
        if value in nc_ds.variables.keys():
            data = nc_ds.variables[value][:]
            data = np.array(data, dtype=dataType)
            attrs = {}
            for attr in nc_ds.variables[value].ncattrs():
                attrs[attr] = getattr(nc_ds.variables[value], attr)
                if (attr == "_FillValue"):
                    no_data.append(attrs[attr])
            nc_attrs.append(attrs)
            nc_data.append(data)
        else:
            print "%s value error " % value
            sys.exit()
    nc_data = np.array(nc_data)
    nc_ds.close()
    del nc_ds
    return lats, lons, nc_data, no_data, lat_attr, lon_attr, nc_attrs


def switType(data_type):
    if (data_type == "float32" or data_type == "float64"):
        nctype = "f"
    elif (data_type == "int16" or data_type == "int8"):
        nctype = "i"
    return nctype
